@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2016 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2017 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -759,7 +759,7 @@ def get_domain():
     if port and host.endswith(':' + port):
         host = host[:-len(port)-1]
     if mm_cfg.VIRTUAL_HOST_OVERVIEW and host:
-        return host.lower()
+        return websafe(host.lower())
     else:
         # See the note in Defaults.py concerning DEFAULT_URL
         # vs. DEFAULT_URL_HOST.
@@ -1273,10 +1273,7 @@ def _DMARCProhibited(mlist, email, dmarc_domain, org=False):
               email, dmarc_domain, e.__doc__)
         return 'continue'
     else:
-# people are already being dumb, don't trust them to provide honest DNS
-# where the answer section only contains what was asked for, nor to include
-# CNAMEs before the values they point to.
-        full_record = ""
+        # Be as robust as possible in parsing the result.
         results_by_name = {}
         cnames = {}
         want_names = set([dmarc_domain + '.'])
@@ -1316,7 +1313,7 @@ def _DMARCProhibited(mlist, email, dmarc_domain, org=False):
                 syslog('error',
                        """RRset of TXT records for %s has %d v=DMARC1 entries;
                        testing them all""",
-                        dmarc_domain, len(dmarc))
+                        dmarc_domain, len(dmarcs))
             for entry in dmarcs:
                 mo = re.search(r'\bsp=(\w*)\b', entry, re.IGNORECASE)
                 if org and mo:
@@ -1394,7 +1391,7 @@ fact is returned."""
                 del recentMemberPostings[addr]
     if not mlist.isMember(email):
         return False
-    return (len(recentMemberPostings.get(email, [])) >=
+    return (len(recentMemberPostings.get(email, [])) >
                 mlist.member_verbosity_threshold
            )
 
